"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Store = exports.Shard = void 0;
const tslib_1 = require("tslib");
const buffer_1 = require("buffer");
const cid_1 = require("multiformats/cid");
const it_drain_1 = (0, tslib_1.__importDefault)(require("it-drain"));
const lru_cache_1 = (0, tslib_1.__importDefault)(require("lru-cache"));
class Shard {
    constructor(ipfs, cid, storeContext) {
        this.ipfs = ipfs;
        this.cid = cid;
        this.storeContext = storeContext;
    }
    static encode(data) {
        return buffer_1.Buffer.from(JSON.stringify(data), 'utf-8');
    }
    static decode(data) {
        return JSON.parse(data.toString('utf-8'));
    }
    static from(ipfs, cid, storeContext) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return new Shard(ipfs, cid, storeContext);
        });
    }
    static create(ipfs, data, storeContext) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            const cid = yield ipfs.object.put({ Data: Shard.encode(data), Links: [] });
            return new Shard(ipfs, cid, storeContext);
        });
    }
    get() {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            const pbNode = yield this.ipfs.object.get(this.cid);
            if (!pbNode.Data)
                return null;
            return Shard.decode(pbNode.Data);
        });
    }
    refreshStore() {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            if (this.storeContext) {
                yield this.storeContext.store.setDHT(this.storeContext.key, this.cid);
            }
        });
    }
    set(data) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            this.cid = yield this.ipfs.object.patch.setData(this.cid, Shard.encode(data));
            yield this.refreshStore();
        });
    }
    /**
     * **Links are EXPERIMENTAL**
     */
    links() {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            const links = yield this.ipfs.object.links(this.cid);
            return Object.fromEntries(links.map(({ Name, Hash }) => [Name, Hash]));
        });
    }
    /**
     * **Links are EXPERIMENTAL**
     */
    addLink(name, cid) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            this.cid = yield this.ipfs.object.patch.addLink(this.cid, {
                Name: name,
                Hash: cid
            });
            yield this.refreshStore();
        });
    }
    /**
     * **Links are EXPERIMENTAL**
     */
    rmLink(cid) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            this.cid = yield this.ipfs.object.patch.rmLink(this.cid, {
                Hash: cid
            });
            yield this.refreshStore();
        });
    }
}
exports.Shard = Shard;
class Store {
    constructor(ipfs, id, cache = { max: 1000000, maxAge: 1000 }) {
        this.ipfs = ipfs;
        this.id = id;
        this.cache = new lru_cache_1.default(cache);
    }
    getDHTKey(key) {
        return buffer_1.Buffer.from(`${this.id}/${key}`);
    }
    setDHT(key, value) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            yield (0, it_drain_1.default)(this.ipfs.dht.put(this.getDHTKey(key), value.bytes, { minPeers: 0 }));
        });
    }
    set(key, data) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            const shard = yield Shard.create(this.ipfs, data, { key, store: this });
            this.cache.set(key, shard.cid);
            yield this.setDHT(key, shard.cid);
            return shard;
        });
    }
    get(key, timeout = 1000) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            // eslint-disable-next-line no-async-promise-executor
            return new Promise((resolve, reject) => (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
                var e_1, _a;
                const cached = this.cache.get(key);
                if (cached)
                    return resolve(Shard.from(this.ipfs, cached, { key, store: this }));
                const interval = setTimeout(() => reject('timeout'), timeout);
                try {
                    for (var _b = (0, tslib_1.__asyncValues)(this.ipfs.dht.get(this.getDHTKey(key))), _c; _c = yield _b.next(), !_c.done;) {
                        const event = _c.value;
                        if (event.name === "VALUE") {
                            clearTimeout(interval);
                            const cid = cid_1.CID.decode(event.value);
                            this.cache.set(key, cid);
                            resolve(Shard.from(this.ipfs, cid, { key, store: this }));
                        }
                    }
                }
                catch (e_1_1) { e_1 = { error: e_1_1 }; }
                finally {
                    try {
                        if (_c && !_c.done && (_a = _b.return)) yield _a.call(_b);
                    }
                    finally { if (e_1) throw e_1.error; }
                }
            }));
        });
    }
}
exports.Store = Store;
//# sourceMappingURL=store.js.map